#include "FastLED.h"

#define NUM_LEDS 60
#define DATA_PIN 6

CRGB leds[NUM_LEDS];

const int buttonPin = 2;    // the pin that the pushbutton is attached to
const int numModes = 15;

int buttonPushCounter = 0;   // counter for the number of button presses
int buttonState = 0;         // current state of the button
int lastButtonState = 0;     // previous state of the button

void setup() {
  pinMode(buttonPin, INPUT);
  Serial.begin(9600);

  // Delay prevents voltage surge against led strip
  delay(2000);
  FastLED.addLeds<WS2812B, DATA_PIN, GRB>(leds, NUM_LEDS);
}

bool checkButtonState() {
  buttonState = digitalRead(buttonPin);

  bool changed = false;

  if (buttonState != lastButtonState) {
    if (buttonState == HIGH) {
      buttonPushCounter++;
      changed = true;
      if (buttonPushCounter == numModes) {
        buttonPushCounter = 0;        
      }
      Serial.println("on");
    } else {
      Serial.println("off");
    }
    delay(50);
  }
  lastButtonState = buttonState; 

  return changed;
}

void loop() {
  
  switch(buttonPushCounter) {
    case 0: fadeInOut(0, 2); break;
    case 1: growingRecedingBars(0, 2); break;
    case 2: oneAtATime(0, 2); break;
    case 3: solidRed(); break;
    case 4: solidGreen(); break;
    case 5: solidBlue(); break;
    case 6: fadeInOut(0, 0); break;
    case 7: fadeInOut(1, 1); break;
    case 8: fadeInOut(2, 2); break;
    case 9: growingRecedingBars(0, 0); break;
    case 10: growingRecedingBars(1, 1); break;
    case 11: growingRecedingBars(2, 2); break;
    case 12: oneAtATime(0, 0); break;
    case 13: oneAtATime(1, 1); break;
    case 14: oneAtATime(2, 2); break;
  }

}

void fadeInOut(int sta, int en) {
  // Fade in/fade out
  for(int j = sta; j <= en; j++ ) { 
    memset(leds, 0, NUM_LEDS * 3);
    for(int k = 0; k < 256; k++) { 
      for(int i = 0; i < NUM_LEDS; i++ ) {
        switch(j) { 
          case 0: leds[i].r = k; break;
          case 1: leds[i].g = k; break;
          case 2: leds[i].b = k; break;
        }
      }
      FastLED.show();
      delay(3);
      if (checkButtonState()) { return; }
    }
    for(int k = 255; k >= 0; k--) { 
      for(int i = 0; i < NUM_LEDS; i++ ) {
        switch(j) { 
          case 0: leds[i].r = k; break;
          case 1: leds[i].g = k; break;
          case 2: leds[i].b = k; break;
        }
      }
      FastLED.show();
      delay(3);
      if (checkButtonState()) { return; }
    }
  }
}

void oneAtATime(int sta, int en) {
  // one at a time
  for(int j = sta; j <= en; j++) { 
    for(int i = 0 ; i < NUM_LEDS; i++ ) {
      memset(leds, 0, NUM_LEDS * 3);
      switch(j) { 
        case 0: leds[i].r = 255; break;
        case 1: leds[i].g = 255; break;
        case 2: leds[i].b = 255; break;
      }
      FastLED.show();
      delay(10);
      if (checkButtonState()) { return; }
    }
  }  
}

void growingRecedingBars(int sta, int en) {
  for(int j = sta; j <= en; j++) { 
    memset(leds, 0, NUM_LEDS * 3);
    for(int i = 0 ; i < NUM_LEDS; i++ ) {
      switch(j) { 
        case 0: leds[i].r = 255; break;
        case 1: leds[i].g = 255; break;
        case 2: leds[i].b = 255; break;
      }
      FastLED.show();
      delay(10);
      if (checkButtonState()) { return; }
    }
    for(int i = NUM_LEDS-1 ; i >= 0; i-- ) {
      switch(j) { 
        case 0: leds[i].r = 0; break;
        case 1: leds[i].g = 0; break;
        case 2: leds[i].b = 0; break;
      }
      FastSPI_LED.show();
      delay(1);
      if (checkButtonState()) { return; }
    }
  }  
}

void solidRed() {
  memset(leds, 0, NUM_LEDS * 3);
  for(int i = 0; i < NUM_LEDS; i++ ) {
    leds[i].r = 255;
    leds[i].g = 0;
    leds[i].b = 0;
  }
  FastLED.show();
  delay(50);
  if (checkButtonState()) { return; }
}

void solidGreen() {
  memset(leds, 0, NUM_LEDS * 3);
  for(int i = 0; i < NUM_LEDS; i++ ) {
    leds[i].r = 0;
    leds[i].g = 255;
    leds[i].b = 0;
  }
  FastLED.show();
  delay(50);
  if (checkButtonState()) { return; }
}

void solidBlue() {
  memset(leds, 0, NUM_LEDS * 3);
  for(int i = 0; i < NUM_LEDS; i++ ) {
    leds[i].r = 0;
    leds[i].g = 0;
    leds[i].b = 255;
  }
  FastLED.show();
  delay(50);
  if (checkButtonState()) { return; }
}

